# Tuesday, July 14th
# functions to fit increasing single index model MLE
# the two main functions are find.mle and search.mle
# find.mle is exact but only works in 2D
# search.mle implements a stochastic search and works for any dimension (matches results from 2D)
# profile.mle implements a version where the index parameter is estimated separately using sqrt(n) consistent methods [from "np"]

library(Iso)
library(rgl)
library(movMF)
library(np)

find.loglik	<-	function(mle, y, nn){
	
	ll1			<-	nn*y*log(mle)
	ll1[y==0]	<-	0
	ll2			<-	nn*(1-y)*log(1-mle)
	ll2[y==1]	<-	0
	ll			<- 	sum(ll1+ll2)/sum(nn)

}

find.pMLE	<- function(alpha, y, x, nn){	

	delta			<-	x %*% alpha
	tt				<-	order(delta)
	mle				<-	pava(y[tt],nn[tt])
	ll				<- 	find.loglik(mle, y[tt], nn[tt])

	return(list(ll=ll, ord=tt, mle=mle))
}

tau			<-	function(r){
	
	k	<-	length(r)
	tau	<-	0
	for(i in 1:(k-1)){
		tau	<-	tau+sum(r[i]>r[(i+1):k])		
	}
	
	return(tau)
	
}


my.inv	<-	function(z){
	
	if((z[1]>=0)*(z[2]>=0)) {theta <- asin(z[2])}
	if((z[1]<0)*(z[2]>=0)) {theta <- acos(z[1])}
	if((z[1]<0)*(z[2]<0)) {theta <- 2*pi-acos(z[1])}
	if((z[1]>=0)*(z[2]<0)) {theta <- 2*pi+asin(z[2])}

	return(theta)	
}

getkmax		<-	function(x, k){
	
	y		<-	sort(x, decreasing=TRUE)
	kmax		<-	y[k]
	locs		<-	which(x >= kmax)
	
	return(locs)
}


# ---------------------------

search.mle		<-	function(x, y, nn, B=10000, k=100, kappa0=100, tol=1e-10, max.iter=20, print=TRUE, doplot=FALSE){
	
	m			<-	length(x[,1])
	d			<-	length(x[1,])
	
	
	if(d!=3)	doplot	<-	FALSE


	alpha		<-	matrix(rnorm(d*B), B, d)
	alpha		<-	alpha/apply(alpha,1, function(x) sqrt(sum(x^2)))

	if(((doplot==TRUE)*(d==3))==1){
	plot3d(alpha, xlim=c(-1,1), ylim=c(-1,1), zlim=c(-1,1))
	}

	loglik		<-	rep(0, B)
	for(i in 1:B){
		res				<-	find.pMLE(alpha[i,], y , x, nn)	
		loglik[i]		<-	res$ll			
		}
	loc			<-	which.max(loglik)
	alphahat		<-	alpha[loc,]
	llhat		<-	loglik[loc]

	check		<-	10

	for(j in 1:max.iter){
	
		if(print==TRUE) print(check)	
		if(check < tol) break	
	
		if(print==TRUE) print(j)

		locs		<-	getkmax(loglik, k)
		if(((doplot==TRUE)*(d==3))==1){
		plot3d(alpha[locs,], xlim=c(-1,1), ylim=c(-1,1), zlim=c(-1,1))
		}

		BB			<-	B/k
		alpha2		<-	matrix(0,BB*k, d)
		kappa		<-	kappa0*(j^2)

		for(i in 1:k){

			mm						<-	alpha[locs[i],]
			temp					<-	rmovMF(BB-1, theta=kappa*mm)
			alpha2[(i-1)*BB+1:BB,]	<-	rbind(mm, temp)
			
		}

		if(((doplot==TRUE)*(d==3))==1){
		plot3d(alpha2, xlim=c(-1,1), ylim=c(-1,1), zlim=c(-1,1), alpha=0.1)
		plot3d(alpha[locs,], xlim=c(-1,1), ylim=c(-1,1), zlim=c(-1,1), add=TRUE, col="red")
		}

		loglik2		<-	rep(0, B)
		for(i in 1:B){
			res				<-	find.pMLE(alpha2[i,], y , x, nn)	
			loglik2[i]		<-	res$ll			
			}
		llhat2		<-	max(loglik2)
		check		<-	(llhat2-llhat)/abs(llhat)

		if(print==TRUE) print(c(llhat, llhat2, check))

		loglik		<-	loglik2
		alpha		<-	alpha2
		llhat		<-	llhat2

		}

	loc			<-	which(loglik==max(loglik))
	alphahat	<-	alpha[loc,]

	if(length(loc)>1)	alphahat1	<-	alphahat[1,]
	else	alphahat1	<-	alphahat
	mle			<-	find.pMLE(alphahat1, y , x, nn)	
	zhat		<-	x %*% alphahat1

	return(list(alphahat=alphahat, mle=mle$mle, zhat=sort(as.vector(zhat))))
		
}

find.mle	<-	function(x, y, nn){
	
	m		<-	length(x[,1])

	theta	<-	matrix(NA,m,m)
	for(i in 1:(m-1)){
		for(j in (i+1):m){
			d			<-	x[i,]-x[j,]
			d			<-	d/sqrt(sum(d^2))
			z			<-	c(-d[2], d[1])
			theta[i,j]	<-	my.inv(z)
			}	
	}
	
	phi		<-	as.vector(theta)
	phi		<-	phi[!is.na(phi)]
	phi		<-	c(phi, (phi+pi) %% (2*pi))
	phi		<-	sort(phi)
	phi		<-	unique(phi)
	
	beta		<-	phi+c(phi[2:length(phi)], phi[1]+2*pi)
	beta		<-	(beta/2) %% (2*pi)

	loglik	<-	rep(0, length(beta))

	for(i in 1:length(beta)){
		alpha		<-	c(cos(beta[i]), sin(beta[i]))
		res			<-	find.pMLE(alpha, y , x, nn)	
		loglik[i]	<-	res$ll	
	}

	loc			<-	which(loglik==max(loglik))
	thetahat	<-	beta[loc]
	thetahat	<-	thetahat %% (2*pi)
	alphahat	<-	matrix(c(cos(thetahat), sin(thetahat)), ncol=2)
	
 	phat	<-	matrix(0, nrow=length(loc), m)
 	ordhat	<-	matrix(0, nrow=length(loc), m)
# 	yordhat	<-	matrix(0, nrow=length(loc), m)
 	for(i in 1:length(loc)){
		alpha		<-	alphahat[i,]
		mle			<-	find.pMLE(alpha, y , x, nn)
		phat[i,]	<-	mle$mle	
 		ordhat[i,]	<-	mle$ord	
# 		yordhat[i,]	<-	y[mle$ord]
 	}
#	phat	<-	unique(phat)
#	phat	<-	as.vector(unique(phat))
	
	return(list(phat=phat, alphahat=alphahat, thetahat=thetahat, ord=ordhat))
}

profile.mle	<-	function(x, y, nn){
	
	m		<-	length(x[,1])

	theta	<-	matrix(NA,m,m)
	for(i in 1:(m-1)){
		for(j in (i+1):m){
			d			<-	x[i,]-x[j,]
			d			<-	d/sqrt(sum(d^2))
			z			<-	c(-d[2], d[1])
			z			<-	sign(z[1])*z
			theta[i,j]	<-	asin(z[2])
			}	
	}
	
	phi		<-	as.vector(theta)
	phi		<-	phi[!is.na(phi)]
	phi		<-	sort(phi)
	phi		<-	c(phi+pi, phi+2*pi)
	
	beta	<-	phi+c(phi[2:length(phi)], phi[1]+2*pi)
	beta	<-	beta/2

	loglik	<-	rep(0, length(beta))

	for(i in 1:length(beta)){
		alpha		<-	c(cos(beta[i]), sin(beta[i]))
		res			<-	find.pMLE(alpha, y , x, nn)	
		loglik[i]	<-	res$ll	
	}

	loc			<-	which(loglik==max(loglik))
	thetahat	<-	beta[loc]
	thetahat	<-	thetahat %% (2*pi)
	alphahat	<-	matrix(c(cos(thetahat), sin(thetahat)), ncol=2)
	
 	phat	<-	matrix(0, nrow=length(loc), m)
 	ordhat	<-	matrix(0, nrow=length(loc), m)
# 	yordhat	<-	matrix(0, nrow=length(loc), m)
 	for(i in 1:length(loc)){
		alpha		<-	alphahat[i,]
		mle			<-	find.pMLE(alpha, y , x, nn)
		phat[i,]	<-	mle$mle	
 		ordhat[i,]	<-	mle$ord	
# 		yordhat[i,]	<-	y[mle$ord]
 	}
#	phat	<-	unique(phat)
#	phat	<-	as.vector(unique(phat))
	
	return(list(phat=phat, alphahat=alphahat, thetahat=thetahat, ord=ordhat))
}


### some preliminary tests of function

#d		<-	10
#m		<-	1000  # num of locations
#nn		<-	sample(100*(1:4), m, replace=TRUE) # number of trials at each location
##n		<-	sum(nn)  # total number of observations
#x		<-	matrix(runif(d*m),m,d)  # random xi in [0,1]^d
#alpha0	<-	rep(1,d) # true alpha
#alpha0	<-	alpha0/sqrt(sum(alpha0^2))
#delta0	<-	x %*% alpha0
#tt0		<-	order(delta0)
#x		<-	x[tt0,]
#delta0	<-	sort(delta0)  
#tau0	<-	rank(delta0)
#mu		<-	pmax(0.5, sin(delta0))
#mu		<-	(delta0/max(delta0))^2
#y		<-	rbinom(m, size=nn, prob=mu)/nn

#mle		<-	search.mle(x, y, nn)

#plot(delta0, mu, type="l")
#lines(mle$zhat, mle$mle, col="red")

#mle$alpha
#alpha0
#if(length(mle$alpha)==d)   acos(sum(alpha0*mle$alpha)) else {apply(mle$alpha, 1, function(x) acos(sum(alpha0*x)))   }

#find.mle(x, y, nn)	
