# Wednesday, February 20th
# R code for data analysis to match Section 4.2 in Manuscript

# read in functions/packages
source("SIMfast.R")
library(np)
library("rgl")
library("bestglm")
set.seed(20-02-2019)

# read in data 
data		<-	read.table("data.txt", header=TRUE)
xx		<-	t(rbind(data$MMS/10, data$PMA))
yy		<-	data$dead/data$total
nn		<-	data$total


# 3D plot of the data set 
plot3d(x=xx[,1], y=xx[,2],z=yy, xlab="MMS", ylab="PMA", zlab="observed death rate", type="s", col="blue", radius=2)


# ------------------ #
# PARAMETRIC MODEL (MODEL 1)
# ------------------ #

# version from Carter et al. (1986) where backward model selection was performed; this is the selected model

x1		<-	rep(data$MMS/10, data$total)
x2		<-	rep(data$PMA, data$total)
x11		<-	x1^2
x22		<-	x2^2
x12		<-	x1*x2
y		<-	numeric()
for(i in 1:16){
	t	<-	c(data$dead[i], data$viable[i]) 	
	y	<-	c(y, rep(c(1,0),t))
	}

model1     <-	glm(y ~ x1 + x2 + x11 +x22, family=binomial)
summary(model1)


# the same model is also selected using AIC and BIC
bestglm(data.frame(x1=x1, x2=x2, x11=x11, x12=x12, x22=x22, y=y), family=binomial, IC="BIC")
bestglm(data.frame(x1=x1, x2=x2, x11=x11, x12=x12, x22=x22, y=y), family=binomial, IC="AIC")

# mean squared error
f1 			<- 	function(x,y) {betahat[1]+betahat[2]*x+betahat[3]*y+betahat[4]*x^2+betahat[5]*y^2}
f1.lm		<-	rep(0,16)
for(i in 1:16)   f1.lm[i]	<-	f1(xx[i,1], xx[i,2])
f1.lm	<-	exp(f1.lm)/(1+exp(f1.lm))

sum((yy-f1.lm)^2)

# ------------------------------------------ #
# NONPARAMETRIC (KERNEL) SINGLE INDEX MODEL (MODEL 2)
# ------------------------------------------ #


x1		<-	rep(data$MMS/10, data$total)
x2		<-	rep(data$PMA, data$total)
y		<-	numeric()
for(i in 1:16){
	t	<-	c(data$dead[i], data$viable[i]) 	
	y	<-	c(y, rep(c(1,0),t))
	}


# find nonparametric bandwidth first (using Klein and Spady method)
# using ten different starting points
bw				<-	npindexbw(xdat=cbind(x1, x2), ydat=as.vector(y), method="kleinspady", nmulti=10)
# fitted value of alpha with convention that first entry is one
bw$beta
# fitted value of alpha with convention that Euclidean norm is one
bw$beta/sqrt(sum(bw$beta^2))

model2			<-	npindex(bws=bw)
summary(model2)




# ---------------------------- #
# ISOTONIC SINGLE INDEX MODEL WITH LINEAR CONTOURS (MODEL 3)
# ---------------------------- #


model3			<-	find.mle(xx, yy, nn)
# fitted value of alpha
model3$alphahat
# fitted values of mu_0
model3$phat


model3b			<-	search.mle(cbind(1,xx), yy, nn, print=FALSE)
# fitted value of alpha
model3b$alphahat
summary(model3b$alphahat[,1])
# fitted values of mu_0
model3b$mle

# mean squared error
sum((yy[model3$ord]-model3$phat)^2)



# ---------------------------- #
# ISOTONIC SINGLE INDEX MODEL WITH ELLIPTICAL CONTOURS (MODEL 4)
# ---------------------------- #

xx4				<-	cbind(xx, xx[,1]^2, xx[,1]*xx[,2], xx[,2]^2)
model4			<-	search.mle(xx4, yy, nn, print=FALSE)
# fitted value of alpha
model4$alphahat
# fitted values of mu_0
model4$mle

# mean squared error
sum((yy[order(xx4 %*% model4$alphahat[1,])]-model4$mle)^2)


model4b			<-	search.mle(cbind(1, xx4), yy, nn, print=FALSE, B=20000)

# fitted value of alpha
model4b$alphahat
summary(model4b$alphahat[,1])
# fitted values of mu_0
model4b$mle


# ---------------------------- #
# ISOTONIC SINGLE INDEX MODEL WITH ELLIPTICAL CONTOURS (parallel to axes, MODEL 5)
# ---------------------------- #

xx5				<-	cbind(xx, xx[,1]^2,  xx[,2]^2)
model5			<-	search.mle(xx5, yy, nn, print=FALSE)
# fitted value of alpha
model5$alphahat
# fitted values of mu_0
model5$mle

# mean squared error
sum((yy[order(xx5 %*% model5$alphahat[1,])]-model5$mle)^2)

model5b		<-	search.mle(cbind(1, xx5), yy, nn, print=FALSE)
# fitted value of alpha
model5b$alphahat
summary(model5b$alphahat[,1])
# fitted values of mu_0
model5b$mle






# ------------------------------------------ #
# HEAT MAPS  (FIGURE 3 in main manuscript)
# ------------------------------------------ #


# parametric logistic regression (model 1)

nlat		<-	101
xx1 		<- 	seq(-1,26,length.out=nlat)
xx2			<-	seq(-1,110,length.out=nlat)
betahat		<-	model1$coeff
f1 			<- 	function(x,y) {betahat[1]+betahat[2]*x+betahat[3]*y+betahat[4]*x^2+betahat[5]*y^2}
zz_lm		<-	outer(xx1,xx2, FUN="f1")
lev0		<-	c(0.25,0.5,0.75,0.95)


#fit.lm	<-	rep(0,length(model3$phat))
#for(i in 1:length(model3$phat))   fit.lm[i]	<-	f1(xx[i,1], xx[i,2])
#fit.lm	<-	exp(fit.lm)/(1+exp(fit.lm))

pdf("cells_model1.pdf", width=3, height=6)
par(las = 1, oma = c(0, 0, 0, 0), cex = 0.8, mar = c(2, 2.5, 1, 1))
plot(0, 0, type = "n", xlim = c(0,25), xlab = "MMS", xaxt = "n", yaxt="n", ylim = c(3,105), ylab = "PMA", main = "")
axis(side=1, at = unique(xx[,1]), labels=c(0,"",10,25))
axis(side=2, at = unique(xx[,2]), labels=c(0,"",10,100))
image(xx1, xx2, zz_lm,levels=seq(0,1, length.out=21), add=TRUE, col  = gray(seq(0.35,0.95, length.out=21)))
contour(xx1, xx2, zz_lm, axes=TRUE, levels=lev0, labels=lev0, frame.plot=TRUE, lwd=1, add=TRUE, labcex=1)
points(xx, pch=19, cex=2*yy)
dev.off()

pdf("cells_model1_zoom.pdf", width=8, height=7)
par(las = 1, oma = c(0, 0, 0, 0), cex = 0.8, mar = c(2, 2.5, 1, 1))
plot(0, 0, type = "n", xlim = c(0,25), xlab = "MMS", xaxt = "n", yaxt="n", ylim = c(0,15), ylab = "PMA", main = "")
axis(side=1, at = unique(xx[,1]), labels=c(0,"",10,25))
axis(side=2, at = unique(xx[,2]), labels=c(0,"",10,100))
image(xx1, xx2, zz_lm,levels=seq(0,1, length.out=21), add=TRUE, col  = gray(seq(0.35,0.95, length.out=21)))
contour(xx1, xx2, zz_lm, axes=TRUE, levels=lev0, labels=lev0, frame.plot=TRUE, lwd=1, add=TRUE, labcex=1)
points(xx, pch=19, cex=2*yy)
dev.off()


# nonparametric (kernel) single index model (model 2)
# WARNING:  the current implementation takes a few minutes to run
# plot not included in paper

nlat		<-	101
xx1 		<- 	seq(-1,26,length.out=nlat)
xx2			<-	seq(-1,110,length.out=nlat)
f2 			<- 	function(t1,t2) {
				f	<-	fitted(npindex(exdat=cbind(t1,t2), eydat=rep(1, 1), bws=bw))
				return(f)
				}
vf			<-	Vectorize(f2)
zz_np		<-	outer(xx1, xx2, FUN="vf")
lev0		<-	c(0.25,0.5,0.75,0.95)

par(las = 1, oma = c(0, 0, 0, 0), cex = 0.8, mar = c(2, 2.5, 1, 1))
plot(0, 0, type = "n", xlim = c(0,25), xlab = "MMS", xaxt = "n", yaxt="n", ylim = c(3,105), ylab = "PMA", main = "")
axis(side=1, at = unique(xx[,1]), labels=c(0,"",10,25))
axis(side=2, at = unique(xx[,2]), labels=c(0,"",10,100))
image(xx1, xx2, zz_np, levels=seq(0,1, length.out=21), add=TRUE, col  = gray(seq(0.35,0.95, length.out=21)))
contour(xx1, xx2, zz_np, axes=TRUE, levels=lev0, labels=lev0, frame.plot=TRUE, lwd=1, add=TRUE, labcex=1)
points(xx, pch=19, cex=2*yy)


# isotonic single index with linear contours (model 3)

alpha.hat	<-	as.vector(model3$alphahat)
fhat			<-	model3$phat[1,]
zhat			<-	xx %*% alpha.hat
ord			<-	order(as.vector(zhat))
zhat			<-	sort(as.vector(zhat))
lev0		<-	c(0.25,0.5,0.75,0.95)


k			<-	length(fhat)
f0			<-	function(t){
	if(t<=zhat[1]) 		f0		<-	fhat[1]
	if(t>=zhat[k])		f0		<-	fhat[k]
	if(((t>zhat[1])*(t<zhat[k]))==1){
						loc		<-	which.max(t<=zhat)
						delta	<-	(t-zhat[loc-1])/(zhat[loc]-zhat[loc-1])
						f0		<-	fhat[loc-1]*(1-delta)+fhat[loc]*delta	
	}
	return(f0)	
}

ffhat	<-	rep(0, length(yy))
for(i in 1:k) ffhat[i] <- f0(alpha.hat[1]*xx[i,1]+alpha.hat[2]*xx[i,2])

nlat		<-	101
xx1 		<- 	seq(-1,26,length.out=nlat)
xx2		<-	seq(-1,110,length.out=nlat)
f 		<- 	function(t1,t2) {
				t	<-	alpha.hat[1]*t1+alpha.hat[2]*t2
				f	<-	f0(t)
				return(f)
				}
vf		<-	Vectorize(f)
zz_sim		<-	outer(xx1, xx2, FUN="vf")

pdf("cells_model3.pdf", width=3, height=6)
par(las = 1, oma = c(0, 0, 0, 0), cex = 0.8, mar = c(2, 2.5, 1, 1))
plot(0, 0, type = "n", xlim = c(0,25), xlab = "MMS", xaxt = "n", yaxt="n", ylim = c(3,105), ylab = "PMA", main = "")
axis(side=1, at = unique(xx[,1]), labels=c(0,"",10,25))
axis(side=2, at = unique(xx[,2]), labels=c(0,"",10,100))
image(xx1, xx2, zz_sim,levels=seq(0,1, length.out=21), add=TRUE, col  = gray(seq(0.35,0.95, length.out=21)))
contour(xx1, xx2, zz_sim, axes=TRUE, levels=lev0, labels=lev0, frame.plot=TRUE, lwd=1, add=TRUE, labcex=1)
points(xx, pch=19, cex=2*yy)
dev.off()


pdf("cells_model3_zoom.pdf", width=8, height=7)
par(las = 1, oma = c(0, 0, 0, 0), cex = 0.8, mar = c(2, 2.5, 1, 1))
plot(0, 0, type = "n", xlim = c(0,25), xlab = "MMS", xaxt = "n", yaxt="n", ylim = c(0,15), ylab = "PMA", main = "")
axis(side=1, at = unique(xx[,1]), labels=c(0,"",10,25))
axis(side=2, at = unique(xx[,2]), labels=c(0,"",10,100))
image(xx1, xx2, zz_sim,levels=seq(0,1, length.out=21), add=TRUE, col  = gray(seq(0.35,0.95, length.out=21)))
contour(xx1, xx2, zz_sim, axes=TRUE, levels=lev0, labels=lev0, frame.plot=TRUE, lwd=1, add=TRUE, labcex=1)
points(xx, pch=19, cex=2*yy)
dev.off()



# isotonic single index with elliptical contours (model 4)

alpha.hat	<-	as.vector(model4$alphahat[1,])
fhat		<-	model4$mle
zhat		<-	xx %*% alpha.hat
ord			<-	order(as.vector(zhat))
zhat		<-	sort(as.vector(zhat))
lev0		<-	c(0.25,0.5,0.75,0.95)


k			<-	length(fhat)
f0			<-	function(t){
	if(t<=zhat[1]) 		f0		<-	fhat[1]
	if(t>=zhat[k])		f0		<-	fhat[k]
	if(((t>zhat[1])*(t<zhat[k]))==1){
						loc		<-	which.max(t<=zhat)
						delta	<-	(t-zhat[loc-1])/(zhat[loc]-zhat[loc-1])
						f0		<-	fhat[loc-1]*(1-delta)+fhat[loc]*delta	
	}
	return(f0)	
}

ffhat	<-	rep(0, length(yy))
for(i in 1:k) ffhat[i] <- f0(alpha.hat[1]*xx[i,1]+alpha.hat[2]*xx[i,2]+alpha.hat[4]*xx[i,1]*xx[i,2]+alpha.hat[3]*xx[i,1]^2+alpha.hat[5]*xx[i,2]^2)


nlat		<-	201
xx1 		<- 	seq(-5,30,length.out=nlat)
xx2			<-	seq(-5,115,length.out=nlat)
f 			<- 	function(t1,t2) {
				t	<-	alpha.hat[1]*t1+alpha.hat[2]*t2+alpha.hat[4]*t1*t2+alpha.hat[3]*t1^2+alpha.hat[5]*t2^2
				f	<-	f0(t)
				return(f)
				}
vf			<-	Vectorize(f)
zz_sim		<-	outer(xx1, xx2, FUN="vf")


pdf("cells_model4.pdf", width=3, height=6)
par(las = 1, oma = c(0, 0, 0, 0), cex = 0.8, mar = c(2, 2.5, 1, 1))
plot(0, 0, type = "n", xlim = c(0,25), xlab = "MMS", xaxt = "n", yaxt="n", ylim = c(3,105), ylab = "PMA", main = "")
axis(side=1, at = unique(xx[,1]), labels=c(0,"",10,25))
axis(side=2, at = unique(xx[,2]), labels=c(0,"",10,100))
image(xx1, xx2, zz_sim,levels=seq(0,1, length.out=21), add=TRUE, col  = gray(seq(0.35,0.95, length.out=21)))
contour(xx1, xx2, zz_sim, axes=TRUE, levels=lev0, labels=lev0, frame.plot=TRUE, lwd=1, add=TRUE, labcex=1)
points(xx, pch=19, cex=2*yy)
dev.off()


pdf("cells_model4_zoom.pdf", width=8, height=7)
par(las = 1, oma = c(0, 0, 0, 0), cex = 0.8, mar = c(2, 2.5, 1, 1))
plot(0, 0, type = "n", xlim = c(0,25), xlab = "MMS", xaxt = "n", yaxt="n", ylim = c(0,15), ylab = "PMA", main = "")
axis(side=1, at = unique(xx[,1]), labels=c(0,"",10,25))
axis(side=2, at = unique(xx[,2]), labels=c(0,"",10,100))
image(xx1, xx2, zz_sim,levels=seq(0,1, length.out=21), add=TRUE, col  = gray(seq(0.35,0.95, length.out=21)))
contour(xx1, xx2, zz_sim, axes=TRUE, levels=lev0, labels=lev0, frame.plot=TRUE, lwd=1, add=TRUE, labcex=1)
points(xx, pch=19, cex=2*yy)
dev.off()


# isotonic single index with elliptical contours parallel to axes (model 5)

alpha.hat	<-	as.vector(model5$alphahat[1,])
fhat		<-	model5$mle
zhat		<-	xx %*% alpha.hat
ord			<-	order(as.vector(zhat))
zhat		<-	sort(as.vector(zhat))
lev0		<-	c(0.25,0.5,0.75,0.95)


k			<-	length(fhat)
f0			<-	function(t){
	if(t<=zhat[1]) 		f0		<-	fhat[1]
	if(t>=zhat[k])		f0		<-	fhat[k]
	if(((t>zhat[1])*(t<zhat[k]))==1){
						loc		<-	which.max(t<=zhat)
						delta	<-	(t-zhat[loc-1])/(zhat[loc]-zhat[loc-1])
						f0		<-	fhat[loc-1]*(1-delta)+fhat[loc]*delta	
	}
	return(f0)	
}

ffhat	<-	rep(0, length(yy))
for(i in 1:k) ffhat[i] <- f0(alpha.hat[1]*xx[i,1]+alpha.hat[2]*xx[i,2]+alpha.hat[3]*xx[i,1]^2+alpha.hat[4]*xx[i,2]^2)


nlat		<-	201
xx1 		<- 	seq(-1,26,length.out=nlat)
xx2			<-	seq(-1,110,length.out=nlat)
f 			<- 	function(t1,t2) {
				t	<-	alpha.hat[1]*t1+alpha.hat[2]*t2+alpha.hat[3]*t1^2+alpha.hat[4]*t2^2
				f	<-	f0(t)
				return(f)
				}
vf			<-	Vectorize(f)
zz_sim		<-	outer(xx1, xx2, FUN="vf")


pdf("cells_model5.pdf", width=3, height=6)
par(las = 1, oma = c(0, 0, 0, 0), cex = 0.8, mar = c(2, 2.5, 1, 1))
plot(0, 0, type = "n", xlim = c(0,25), xlab = "MMS", xaxt = "n", yaxt="n", ylim = c(3,105), ylab = "PMA", main = "")
axis(side=1, at = unique(xx[,1]), labels=c(0,"",10,25))
axis(side=2, at = unique(xx[,2]), labels=c(0,"",10,100))
image(xx1, xx2, zz_sim,levels=seq(0,1, length.out=21), add=TRUE, col  = gray(seq(0.35,0.95, length.out=21)))
contour(xx1, xx2, zz_sim, axes=TRUE, levels=lev0, labels=lev0, frame.plot=TRUE, lwd=1, add=TRUE, labcex=1)
points(xx, pch=19, cex=2*yy)
dev.off()


pdf("cells_model5_zoom.pdf", width=8, height=7)
par(las = 1, oma = c(0, 0, 0, 0), cex = 0.8, mar = c(2, 2.5, 1, 1))
plot(0, 0, type = "n", xlim = c(0,25), xlab = "MMS", xaxt = "n", yaxt="n", ylim = c(0,15), ylab = "PMA", main = "")
axis(side=1, at = unique(xx[,1]), labels=c(0,"",10,25))
axis(side=2, at = unique(xx[,2]), labels=c(0,"",10,100))
image(xx1, xx2, zz_sim,levels=seq(0,1, length.out=21), add=TRUE, col  = gray(seq(0.35,0.95, length.out=21)))
contour(xx1, xx2, zz_sim, axes=TRUE, levels=lev0, labels=lev0, frame.plot=TRUE, lwd=1, add=TRUE, labcex=1)
points(xx, pch=19, cex=2*yy)
dev.off()



# ------------------------------------------ #
# PROJECTION PLOTS  
# ------------------------------------------ #


xx1 			<- 	seq(-10,36,length.out=1001)
xx2			<-	seq(-10,140,length.out=1001)

alpha.iso	<-	sim.mle$alphahat
z.iso		<-	as.vector(cbind(xx1,xx2) %*% t(alpha.iso))
fit.iso 		<- 	sim.mle$phat


alpha.np		<-	bw$beta
alpha.np		<-	alpha.np/sqrt(sum(alpha.np^2))
z.np			<-	as.vector(cbind(xx1,xx2) %*% alpha.np)
fit.np 		<- 	fitted(npindex(exdat=cbind(xx1,xx2), eydat=rep(1, length(xx1)), bws=bw))

betahat		<-	par.mle$coeff
f 			<- 	function(x,y) {betahat[1]+betahat[2]*x+betahat[3]*y+betahat[4]*x^2+betahat[5]*y^2}

xx1 			<- 	seq(-10,36,length.out=101)
xx2			<-	seq(-10,140,length.out=101)

f.lm			<-	matrix(0, 101, 101)
z.iso.lm		<-	matrix(0, 101, 101)
z.np.lm		<-	matrix(0, 101, 101)
for(i in 1:101){
	for(j in 1:101){
		f.lm[i,j] 		<-  f(xx1[i],xx2[j])
		z.iso.lm[i,j]	<-	alpha.iso[1]*xx1[i]+alpha.iso[2]*xx2[j]
		z.np.lm[i,j]	<-	alpha.np[1]*xx1[i]+alpha.np[2]*xx2[j]
	}
}

ff.lm			<-	as.vector(exp(f.lm)/(1+exp(f.lm)))

zz.iso.lm		<-	as.vector(z.iso.lm)
iso.ord			<-	order(zz.iso.lm)
zz.np.lm			<-	as.vector(z.np.lm)
np.ord			<-	order(zz.np.lm)

#pdf("cells_lm_vs_sim.pdf", width=8, height=6)
par(mar=c(2,4,1,1))
plot(as.vector(as.matrix(xx) %*% alpha.hat), yy, pch=19, cex=0.5*nn/max(nn)+1, xlab="<fitted alpha,x>", ylab="death rate", xlim=c(0,105), ylim=c(0,1), col=gray(0.5))            
points(zhat, fhat, type="l", lwd=2, lty=3)
lines(smooth.spline(zz.iso.lm[iso.ord], ff.lm[iso.ord], df = 10), lwd = 2, lty=1)
#dev.off()


#pdf("cells_np_vs_lm.pdf", width=8, height=6)
par(mar=c(2,4,1,1))
plot(as.vector(as.matrix(xx) %*% alpha.np), yy, pch=19, cex=0.5*nn/max(nn)+1, xlab="<fitted alpha,x>", ylab="death rate", xlim=c(0,28), ylim=c(0,1), col=gray(0.5))            
points(sort(z.np), fit.np[order(z.np)], type="l", lwd=2, lty=5)
lines(smooth.spline(zz.np.lm[np.ord], ff.lm[np.ord], df = 10), lwd = 2, lty=1)
#dev.off()

